/// <reference types="node" />
import { Key } from './key';
export interface KleisimoObj<T> {
    readonly e: string[];
    readonly h: string;
}
export interface ValueType<T, H = KleisimoObj<T>> {
    readonly name: string;
    readonly type: string;
    key?: Key;
    set(t: T): T;
    get(): T;
    setKleisimo(t: H): T;
    getKleisimo(): H;
}
export declare class AttributeRegister<O, H> {
    readonly attributes: ValueType<unknown, unknown>[];
    register<T, H>(vt: ValueType<T, H>): void;
    asUnencrypted(): O;
    asEncrypted(): H;
}
export declare class KleisimoAttribute<T> implements ValueType<T> {
    readonly name: string;
    readonly type: string;
    key: Key;
    value: T;
    constructor(ar: AttributeRegister<unknown, unknown>, name: string, type: string, key: Key);
    set(t: T): T;
    get(): T;
    unpad(b: string): string;
    setKleisimo(t: KleisimoObj<T>): T;
    pad(p: string): Buffer;
    getKleisimo(): KleisimoObj<T>;
}
export interface KleisimoDateObj {
    month: number;
    year: number;
    encrypted: string;
}
export declare class KleisimoDateAttribute implements ValueType<Date, KleisimoDateObj> {
    readonly name: string;
    readonly type: string;
    key: Key;
    value: Date;
    constructor(ar: AttributeRegister<unknown, unknown>, name: string, key: Key);
    get(): Date;
    setKleisimo(t: KleisimoDateObj): Date;
    getKleisimo(): KleisimoDateObj;
    set(t: Date): Date;
}
export interface KleisimoZipObj {
    readonly zip: string;
}
export declare class KleisimoZipAttribute implements ValueType<string, KleisimoZipObj> {
    readonly name: string;
    readonly type: string;
    key: Key;
    value: string;
    constructor(ar: AttributeRegister<unknown, unknown>, name: string, key: Key);
    get(): string;
    setKleisimo(t: KleisimoZipObj): string;
    getKleisimo(): KleisimoZipObj;
    set(t: string): string;
}
export declare class OpenAttribute<T> implements ValueType<T, T> {
    readonly name: string;
    readonly type: string;
    value: T;
    constructor(ar: AttributeRegister<unknown, unknown>, name: string, type: string);
    key?: Key | undefined;
    set(t: T): T;
    get(): T;
    getKleisimo(): T;
    setKleisimo(t: T): T;
}
